﻿describe("Announce movement test", ()=> {
    beforeAll(async () => {
        await page.goto(PATH + "mapml-viewer.html");
    });

    afterAll(async function () {
        await context.close();
    });

    test("Output values are correct during regular movement", async ()=>{
        await page.keyboard.press("Tab");
        await page.waitForTimeout(500);
        await page.keyboard.press("ArrowUp");
        await page.waitForTimeout(1000);

        const movedUp = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(movedUp).toEqual("zoom level 0 column 3 row 3");

        for(let i = 0; i < 2; i++){
            await page.keyboard.press("ArrowLeft");
            await page.waitForTimeout(1000);
        }

        const movedLeft = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(movedLeft).toEqual("zoom level 0 column 2 row 3");

        await page.keyboard.press("Equal");
        await page.waitForTimeout(1000);

        const zoomedIn = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(zoomedIn).toEqual("zoom level 1 column 4 row 6");
    });

    test("Output values are correct at bounds and bounces back", async ()=>{
        //Zoom out to min layer bound
        await page.keyboard.press("Minus");
        await page.waitForTimeout(1000);

        const minZoom = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(minZoom).toEqual("At minimum zoom level, zoom out disabled zoom level 0 column 2 row 3");

        //Pan out of west bounds, expect the map to bounce back
        for(let i = 0; i < 4; i++){
            await page.waitForTimeout(1000);
            await page.keyboard.press("ArrowLeft");
        }

        const westBound = await page.waitForFunction(() =>
            document.querySelector("body > mapml-viewer").shadowRoot.querySelector("div > output").innerHTML === "Reached west bound, panning west disabled",
            {}, {timeout: 1000}
        );
        await expect(await westBound.jsonValue()).toEqual(true);

        await page.waitForTimeout(1000);
        const bouncedBack = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(bouncedBack).toEqual("zoom level 0 column 1 row 3");

        //Zoom in out of bounds, expect the map to zoom back
        await page.keyboard.press("Equal");

        const zoomedOutOfBounds = await page.waitForFunction(() =>
            document.querySelector("body > mapml-viewer").shadowRoot.querySelector("div > output").innerHTML === "Zoomed out of bounds, returning to",
            {}, {timeout: 1000}
        );
        await expect(await zoomedOutOfBounds.jsonValue()).toEqual(true);

        await page.waitForTimeout(1000);
        const zoomedBack = await page.$eval(
            "body > mapml-viewer div > output",
            (output) => output.innerHTML
        );
        await expect(zoomedBack).toEqual("zoom level 0 column 1 row 3");

    });
});